@@ -4,11 +4,15 @@
 #include "number.hpp"
 #include "../utils.hpp"
 
+namespace mys {
+
+class String;
+
 // A bytes.
 class Bytes final {
 
 public:
-    std::shared_ptr<std::vector<u8>> m_bytes;
+    mys::shared_ptr<std::vector<u8>> m_bytes;
 
     Bytes() : m_bytes(nullptr)
     {
@@ -18,11 +22,15 @@ public:
     {
     }
 
+    Bytes(u64 size);
+
     Bytes(std::initializer_list<u8> il) :
-        m_bytes(std::make_shared<std::vector<u8>>(il))
+        m_bytes(mys::make_shared<std::vector<u8>>(il))
     {
     }
 
+    Bytes(String hex);
+
     u8& operator[](i64 index) const;
 
     bool operator==(const Bytes& other) const
@@ -51,13 +59,28 @@ public:
         m_bytes->push_back(other);
     }
 
+    i64 find(const Bytes& needle,
+             std::optional<i64> start,
+             std::optional<i64> end) const;
+
     int __len__() const
     {
         return shared_ptr_not_none(m_bytes)->size();
     }
+
+    String to_hex() const;
 };
 
 std::ostream&
 operator<<(std::ostream& os, const Bytes& obj);
 
+#if !defined(MYS_UNSAFE)
 const Bytes& bytes_not_none(const Bytes& obj);
+#else
+static inline const Bytes& bytes_not_none(const Bytes& obj)
+{
+    return obj;
+}
+#endif
+
+}
