@@ -3,6 +3,8 @@
 #include "../common.hpp"
 #include "string.hpp"
 
+namespace mys {
+
 class Object {
 public:
     virtual void __format__(std::ostream& os) const;
@@ -10,3 +12,5 @@ public:
 };
 
 std::ostream& operator<<(std::ostream& os, Object& obj);
+
+}
