@@ -1226,8 +1226,7 @@ tok_get(struct tok_state *tok, const char **p_start, const char **p_end)
                 col++, altcol++;
             }
             else if (c == '\t') {
-                col = (col / tok->tabsize + 1) * tok->tabsize;
-                altcol = (altcol / ALTTABSIZE + 1) * ALTTABSIZE;
+                return syntaxerror(tok, "indentations must be spaces, not tabs");
             }
             else if (c == '\014')  {/* Control-L (formfeed) */
                 col = altcol = 0; /* For Emacs users */
@@ -1268,6 +1267,10 @@ tok_get(struct tok_state *tok, const char **p_start, const char **p_end)
             }
             else if (col > tok->indstack[tok->indent]) {
                 /* Indent -- always one */
+                if ((col - tok->indstack[tok->indent]) != 4) {
+                    tok_nextc(tok);
+                    return syntaxerror(tok, "indentations must be 4 spaces");
+                }
                 if (tok->indent+1 >= MAXINDENT) {
                     tok->done = E_TOODEEP;
                     tok->cur = tok->inp;
@@ -1656,7 +1659,7 @@ tok_get(struct tok_state *tok, const char **p_start, const char **p_end)
                 else if (c == 'e' || c == 'E') {
                     goto exponent;
                 }
-                else if (c == 'j' || c == 'J') {
+                else if (c == 'i' || c == 'I') {
                     goto imaginary;
                 }
                 else if (nonzero) {
@@ -1712,7 +1715,7 @@ tok_get(struct tok_state *tok, const char **p_start, const char **p_end)
                         return ERRORTOKEN;
                     }
                 }
-                if (c == 'j' || c == 'J') {
+                if (c == 'i' || c == 'I') {
                     /* Imaginary part */
         imaginary:
                     c = tok_nextc(tok);
