@@ -1,11 +1,14 @@
 from collections import defaultdict
+from textwrap import indent
 
 from ..parser import ast
 from .utils import INTEGER_TYPES
-from .utils import METHOD_OPERATORS
+from .utils import METHOD_BIN_OPERATORS
 from .utils import CompileError
 from .utils import GenericType
+from .utils import format_mys_type
 from .utils import get_import_from_info
+from .utils import has_docstring
 from .utils import is_pascal_case
 from .utils import is_snake_case
 from .utils import is_upper_snake_case
@@ -26,7 +29,7 @@ class TypeVisitor(ast.NodeVisitor):
         return [self.visit(elem) for elem in node.elts]
 
     def visit_Tuple(self, node):
-        return tuple([self.visit(elem) for elem in node.elts])
+        return tuple(self.visit(elem) for elem in node.elts)
 
     def visit_Dict(self, node):
         return {node.keys[0].id: self.visit(node.values[0])}
@@ -49,6 +52,52 @@ class TypeVisitor(ast.NodeVisitor):
 
         return GenericType(self.visit(node.value), types, node)
 
+
+class FormatDefaultVisitor(ast.NodeVisitor):
+
+    def visit_Constant(self, node):
+        if isinstance(node.value, str):
+            return f'"{node.value}"'
+        elif isinstance(node.value, tuple) and len(node.value) == 3:
+            return f"'{node.value[0]}'"
+        else:
+            return str(node.value)
+
+    def visit_UnaryOp(self, node):
+        operand = self.visit(node.operand)
+
+        if isinstance(node.op, ast.USub):
+            return f'-{operand}'
+        else:
+            return operand
+
+    def visit_List(self, node):
+        return '[' + ', '.join([self.visit(elem) for elem in node.elts]) + ']'
+
+    def visit_Tuple(self, node):
+        return '(' + ', '.join([self.visit(elem) for elem in node.elts]) + ')'
+
+    def visit_Dict(self, node):
+        return '{' + ', '.join([
+            f'{self.visit(key)}: {self.visit(value)}'
+            for key, value in zip(node.keys, node.values)
+        ]) + '}'
+
+    def visit_Set(self, node):
+        return '{' + ', '.join([self.visit(elem) for elem in node.elts]) + '}'
+
+    def visit_Call(self, node):
+        params = ', '.join([self.visit(arg) for arg in node.args])
+
+        return f'{node.func.id}({params})'
+
+    def visit_Name(self, node):
+        return node.id
+
+    def visit_Attribute(self, node):
+        return f'{self.visit(node.value)}.{node.attr}'
+
+
 class Function:
 
     def __init__(self,
@@ -60,7 +109,8 @@ class Function:
                  returns,
                  node,
                  module_name=None,
-                 is_overloaded=False):
+                 is_overloaded=False,
+                 docstring=None):
         self.name = name
         self.generic_types = generic_types
         self.raises = raises
@@ -70,6 +120,7 @@ class Function:
         self.node = node
         self.module_name = module_name
         self.is_overloaded = is_overloaded
+        self.docstring = docstring
 
     def is_generic(self):
         return bool(self.generic_types)
@@ -83,6 +134,43 @@ class Function:
         else:
             return self.name
 
+    def signature_string(self, method):
+        params = []
+
+        for param, default in self.args:
+            param = f'{param.name}: {format_mys_type(param.type)}'
+
+            if default is not None:
+                param += ' = '
+                param += FormatDefaultVisitor().visit(default)
+
+            params.append(param)
+
+        params_string = ', '.join(params)
+
+        if len(params_string) > 60:
+            if method:
+                if params:
+                    params = ['self'] + params
+                else:
+                    params = ['self']
+
+            params_string = indent(',\n'.join(params),
+                                   ' ' * (len(self.name) + 5)).strip()
+        else:
+            if method:
+                if params_string:
+                    params_string = 'self, ' + params_string
+                else:
+                    params_string = 'self'
+
+        if self.returns is None:
+            returns = ''
+        else:
+            returns = f' -> {format_mys_type(self.returns)}'
+
+        return f'{self.name}({params_string}){returns}'
+
     def __str__(self):
         args = [f'{param.name}: {param.type}' for param, _ in self.args]
 
@@ -122,7 +210,8 @@ class Class:
                  functions,
                  implements,
                  node,
-                 module_name=None):
+                 module_name=None,
+                 docstring=None):
         self.name = name
         self.generic_types = generic_types
         self.members = members
@@ -131,6 +220,7 @@ class Class:
         self.implements = implements
         self.node = node
         self.module_name = module_name
+        self.docstring = docstring
 
     def is_generic(self):
         return bool(self.generic_types)
@@ -156,27 +246,34 @@ class Class:
 
 class Trait:
 
-    def __init__(self, name, methods, node):
+    def __init__(self, name, methods, node, docstring):
         self.name = name
         self.methods = methods
         self.node = node
+        self.docstring = docstring
 
 
 class Enum:
 
-    def __init__(self, name, type_, members):
+    def __init__(self, name, type_, members, docstring):
         self.name = name
         self.type = type_
         self.members = members
         self.member_names = {name for name, _ in members}
+        self.docstring = docstring
 
 
 class Variable:
 
-    def __init__(self, name, type_, node):
+    def __init__(self,
+                 name,
+                 type_,
+                 node,
+                 docstring):
         self.name = name
         self.type = type_
         self.node = node
+        self.docstring = docstring
 
     def __str__(self):
         return f'Variable(name={self.name}, type={self.type})'
@@ -355,13 +452,21 @@ class FunctionVisitor(TypeVisitor):
         else:
             returns = FunctionVisitor().visit(node.returns)
 
+        if has_docstring(node):
+            docstring = node.body[0].value.value
+        else:
+            docstring = None
+
         return Function(node.name,
                         decorators.get('generic', []),
                         decorators.get('raises', []),
                         'test' in decorators,
                         args,
                         returns,
-                        node)
+                        node,
+                        None,
+                        None,
+                        docstring)
 
 
 class MethodVisitor(FunctionVisitor):
@@ -447,7 +552,7 @@ def visit_decorator_list(decorator_list, allowed_decorators):
 
 
 def validate_method_signature(method, method_node):
-    if method.name in METHOD_OPERATORS:
+    if method.name in METHOD_BIN_OPERATORS:
         if len(method.args) != 1:
             raise CompileError(
                 f'{method.name} must take exactly one parameter', method_node)
@@ -466,8 +571,18 @@ class DefinitionsVisitor(ast.NodeVisitor):
         self._next_enum_value = None
 
     def visit_Module(self, node):
-        for item in node.body:
-            self.visit(item)
+        body_iter = iter(node.body)
+        item_index = 0
+
+        for item in body_iter:
+            if isinstance(item, ast.AnnAssign):
+                if self.visit_global_variable(item, node.body, item_index + 1):
+                    next(body_iter)
+                    item_index += 1
+            else:
+                self.visit(item)
+
+            item_index += 1
 
         return self._definitions
 
@@ -477,7 +592,17 @@ class DefinitionsVisitor(ast.NodeVisitor):
     def visit_Assign(self, node):
         raise CompileError("global variable types cannot be inferred", node)
 
-    def visit_AnnAssign(self, node):
+    def visit_global_variable(self, node, body, next_index):
+        docstring = None
+
+        if len(body) > next_index:
+            docstring_node = body[next_index]
+
+            if isinstance(docstring_node, ast.Expr):
+                if isinstance(docstring_node.value, ast.Constant):
+                    if isinstance(docstring_node.value.value, str):
+                        docstring = docstring_node.value.value
+
         name = node.target.id
 
         if not is_upper_snake_case(name):
@@ -488,9 +613,12 @@ class DefinitionsVisitor(ast.NodeVisitor):
             name,
             Variable(name,
                      TypeVisitor().visit(node.annotation),
-                     node),
+                     node,
+                     docstring),
             node)
 
+        return docstring is not None
+
     def visit_enum_member_assign(self, node):
         if len(node.targets) != 1:
             raise CompileError("invalid enum member syntax", node)
@@ -572,14 +700,22 @@ class DefinitionsVisitor(ast.NodeVisitor):
 
         self._next_enum_value = None
         members = []
+        body_iter = iter(node.body)
 
-        for item in node.body:
+        if has_docstring(node):
+            docstring = node.body[0].value.value
+            next(body_iter)
+        else:
+            docstring = None
+
+        for item in body_iter:
             members.append(self.visit_enum_member(item))
 
         self._definitions.define_enum(enum_name,
                                       Enum(enum_name,
                                            decorators['enum'],
-                                           members),
+                                           members,
+                                           docstring),
                                       node)
 
     def visit_trait(self, node):
@@ -589,8 +725,15 @@ class DefinitionsVisitor(ast.NodeVisitor):
             raise CompileError("trait names must be pascal case", node)
 
         methods = defaultdict(list)
+        body_iter = iter(node.body)
 
-        for item in node.body:
+        if has_docstring(node):
+            docstring = node.body[0].value.value
+            next(body_iter)
+        else:
+            docstring = None
+
+        for item in body_iter:
             if isinstance(item, ast.FunctionDef):
                 name = item.name
 
@@ -600,7 +743,7 @@ class DefinitionsVisitor(ast.NodeVisitor):
                 raise CompileError('traits cannot have members', item)
 
         self._definitions.define_trait(trait_name,
-                                       Trait(trait_name, methods, node),
+                                       Trait(trait_name, methods, node, docstring),
                                        node)
 
     def visit_class(self, node, decorators):
@@ -618,8 +761,15 @@ class DefinitionsVisitor(ast.NodeVisitor):
             trait.id: trait
             for trait in node.bases
         }
+        body_iter = iter(node.body)
 
-        for item in node.body:
+        if has_docstring(node):
+            docstring = node.body[0].value.value
+            next(body_iter)
+        else:
+            docstring = None
+
+        for item in body_iter:
             if isinstance(item, ast.FunctionDef):
                 name = item.name
 
@@ -650,7 +800,9 @@ class DefinitionsVisitor(ast.NodeVisitor):
                                              methods,
                                              functions,
                                              implements,
-                                             node),
+                                             node,
+                                             None,
+                                             docstring),
                                        node)
 
     def visit_ClassDef(self, node):
@@ -697,7 +849,7 @@ class MakeFullyQualifiedNames:
                 self.process_type(list(mys_type.values())[0])
             }
         elif isinstance(mys_type, tuple):
-            return tuple([self.process_type(item) for item in mys_type])
+            return tuple(self.process_type(item) for item in mys_type)
         elif isinstance(mys_type, GenericType):
             mys_type.name = self.process_type(mys_type.name)
             types = []
