@@ -7,6 +7,7 @@ from .context import Context
 from .generics import TypeVisitor
 from .generics import add_generic_class
 from .generics import format_parameters
+from .return_checker_visitor import ReturnCheckerVisitor
 from .utils import BUILTIN_ERRORS
 from .utils import CompileError
 from .utils import GenericType
@@ -27,12 +28,12 @@ from .value_check_type_visitor import ValueCheckTypeVisitor
 def create_coverage_exit(module_path, coverage_variables):
     code = [
         '#if defined(MYS_COVERAGE)',
-        f'    mys_coverage_file << "File: {module_path}" << "\\n";'
+        f'    mys::mys_coverage_file << "File: {module_path}" << "\\n";'
     ]
 
     for lineno, variable in coverage_variables:
         code.append(
-            f'    mys_coverage_file << {lineno} << " " << {variable} << "\\n";')
+            f'    mys::mys_coverage_file << {lineno} << " " << {variable} << "\\n";')
 
     code.append('#endif')
 
@@ -54,8 +55,9 @@ def create_enum_from_integer(enum):
 
     code += [
         '    default:',
-        '        std::make_shared<ValueError>('
-        f'String("enum {enum.name} does not contain ") + String(value))->__throw();',
+        '        mys::make_shared<ValueError>('
+        f'mys::String("enum {enum.name} does not contain ") + '
+        'mys::String(value))->__throw();',
         '    }',
         '}'
     ]
@@ -74,6 +76,7 @@ class SourceVisitor(ast.NodeVisitor):
                  module_levels,
                  module_hpp,
                  filename,
+                 version,
                  source_lines,
                  definitions,
                  module_definitions,
@@ -84,6 +87,7 @@ class SourceVisitor(ast.NodeVisitor):
         self.module_levels = module_levels
         self.module_hpp = module_hpp
         self.filename = filename
+        self.version = version
         self.skip_tests = skip_tests
         self.namespace = namespace
         self.add_package_main = False
@@ -164,7 +168,7 @@ class SourceVisitor(ast.NodeVisitor):
     def add_application_exit(self, ordered_modules):
         body = [
             '#if defined(MYS_COVERAGE)',
-            '    mys_coverage_file.open(".mys-coverage.txt");',
+            '    mys::mys_coverage_file.open(".mys-coverage.txt");',
             '#endif'
         ]
 
@@ -178,7 +182,7 @@ class SourceVisitor(ast.NodeVisitor):
 
         body += [
             '#if defined(MYS_COVERAGE)',
-            '    mys_coverage_file.close();',
+            '    mys::mys_coverage_file.close();',
             '#endif'
         ]
 
@@ -194,7 +198,7 @@ class SourceVisitor(ast.NodeVisitor):
             raise CompileError(f"undefined type '{mys_type}'", node)
 
     def visit_AnnAssign(self, node):
-        return GlobalVariableVisitor(self.context, '').visit(node)
+        return GlobalVariableVisitor(self.context, '', '').visit(node)
 
     def visit_variable(self, variable):
         mys_type = TypeVisitor(self.context).visit(variable.node.annotation)
@@ -202,9 +206,34 @@ class SourceVisitor(ast.NodeVisitor):
 
         return [f'{cpp_type} {variable.name};']
 
+    def visit_global_variable(self, node, body, next_index):
+        docstring = None
+
+        if len(body) > next_index:
+            docstring_node = body[next_index]
+
+            if isinstance(docstring_node, ast.Expr):
+                if isinstance(docstring_node.value, ast.Constant):
+                    if isinstance(docstring_node.value.value, str):
+                        docstring = docstring_node.value.value
+
+        self.init_globals += GlobalVariableVisitor(self.context, '', '').visit(node)
+
+        return docstring is not None
+
     def visit_Module(self, node):
-        for item in node.body:
-            self.init_globals += self.visit(item)
+        body_iter = iter(node.body)
+        item_index = 0
+
+        for item in body_iter:
+            if isinstance(item, ast.AnnAssign):
+                if self.visit_global_variable(item, node.body, item_index + 1):
+                    next(body_iter)
+                    item_index += 1
+            else:
+                self.init_globals += self.visit(item)
+
+            item_index += 1
 
         for name, definitions in self.module_definitions.classes.items():
             if definitions.is_generic():
@@ -234,13 +263,24 @@ class SourceVisitor(ast.NodeVisitor):
         self.body += self.visit_class_definition(name, definitions)
 
     def format_cpp(self):
+        traceback_entries = ',\n'.join([
+            f'    {{ "{name}", {lineno}, "{code}" }}'
+            for name, lineno, code in self.context.traceback.entries
+        ])
         return '\n'.join([
             '// This file was generated by mys. DO NOT EDIT!!!',
             '#include "mys.hpp"',
             f'#include "{self.module_hpp}"'
         ] + self.before_namespace + [
             f'namespace {self.namespace}',
-            '{'
+            '{',
+            'static mys::TracebackEntryInfo __traceback_entries_info[] = {',
+            traceback_entries,
+            '};',
+            'static mys::TracebackModuleInfo __traceback_module_info = {',
+            f'   .path_p = "{self.filename}",',
+            '   .entries_info_p = &__traceback_entries_info[0]',
+            '};'
         ] + self.in_namespace
           + self.enums
           + [constant[1] for constant in self.context.constants.values()]
@@ -263,7 +303,6 @@ class SourceVisitor(ast.NodeVisitor):
             return [
                 'void package_main(int argc, const char *argv[])',
                 '{',
-                '    core_fiber::init();',
                 f'    {self.namespace}::main(argc, argv);',
                 '}'
             ]
@@ -280,12 +319,17 @@ class SourceVisitor(ast.NodeVisitor):
             key_mys_type, value_mys_type = split_dict_mys_type(mys_type)
             self.define_implicitly_imported_types(key_mys_type)
             self.define_implicitly_imported_types(value_mys_type)
+        elif isinstance(mys_type, GenericType):
+            # ToDo, but what should be done?
+            pass
         elif '.' in mys_type:
             module = '.'.join(mys_type.split('.')[:-1])
             name = mys_type.split('.')[-1]
             imported = self.definitions.get(module)
 
-            if name in imported.classes:
+            if imported is None:
+                raise Exception(f"imported module '{module}' not found")
+            elif name in imported.classes:
                 self.context.define_class(mys_type, mys_type, imported.classes[name])
             elif name in imported.traits:
                 self.context.define_trait(mys_type, mys_type, imported.traits[name])
@@ -363,7 +407,9 @@ class SourceVisitor(ast.NodeVisitor):
 
             cpp_type = mys_to_cpp_type(param.type, self.context)
             body = ValueCheckTypeVisitor(
-                BaseVisitor(self.context, self.filename)).visit(default, param.type)
+                BaseVisitor(self.context,
+                            self.filename,
+                            self.version)).visit(default, param.type)
 
             code += [
                 format_default(name, param.name, cpp_type),
@@ -400,6 +446,10 @@ class SourceVisitor(ast.NodeVisitor):
                 method.node.args.args[0])
             self.define_parameters(method.args)
             self.raise_if_type_not_defined(method.returns, method.node.returns)
+
+            if method.returns is not None:
+                ReturnCheckerVisitor().visit(method.node)
+
             method_names.append(method.name)
             method_name = format_method_name(method, class_name)
             parameters = format_parameters(method.args, self.context)
@@ -413,6 +463,7 @@ class SourceVisitor(ast.NodeVisitor):
                     f'{return_cpp_type} {class_name}::{method_name}({parameters})')
 
             body.append('{')
+            body.append(self.context.traceback.enter(method.name))
             body_iter = iter(method.node.body)
 
             if has_docstring(method.node):
@@ -420,9 +471,12 @@ class SourceVisitor(ast.NodeVisitor):
 
             for item in body_iter:
                 BodyCheckVisitor().visit(item)
+                body.append(self.context.traceback.set(item.lineno))
                 body.append(indent(BodyVisitor(self.context,
-                                               self.filename).visit(item)))
+                                               self.filename,
+                                               self.version).visit(item)))
 
+            body.append(self.context.traceback.exit())
             body.append('}')
             self.context.pop()
 
@@ -458,7 +512,7 @@ class SourceVisitor(ast.NodeVisitor):
             body += [
                 f'void {class_name}::__throw()',
                 '{',
-                f'    throw __{class_name}(shared_from_this());',
+                f'    throw __{class_name}(mys::shared_ptr<{class_name}>(this));',
                 '}'
             ]
 
@@ -473,7 +527,7 @@ class SourceVisitor(ast.NodeVisitor):
     def visit_function_definition_main(self, function, parameters, body):
         self.add_package_main = True
 
-        if parameters not in ['const SharedList<String>& argv', 'void']:
+        if parameters not in ['SharedList<mys::String> argv', 'void']:
             raise CompileError("main() takes 'argv: [string]' or no arguments",
                                function.node)
 
@@ -522,11 +576,15 @@ class SourceVisitor(ast.NodeVisitor):
         self.context.push()
         self.define_parameters(function.args)
         self.raise_if_type_not_defined(function.returns, function.node.returns)
+
+        if function.returns is not None:
+            ReturnCheckerVisitor().visit(function.node)
+
         function_name = function.make_name()
         parameters = format_parameters(function.args, self.context)
         return_cpp_type = format_return_type(function.returns, self.context)
         self.context.return_mys_type = function.returns
-        body = []
+        body = [self.context.traceback.enter(function.name)]
         body_iter = iter(function.node.body)
 
         if has_docstring(function.node):
@@ -534,7 +592,12 @@ class SourceVisitor(ast.NodeVisitor):
 
         for item in body_iter:
             BodyCheckVisitor().visit(item)
-            body.append(indent(BodyVisitor(self.context, self.filename).visit(item)))
+            body.append(self.context.traceback.set(item.lineno))
+            body.append(indent(BodyVisitor(self.context,
+                                           self.filename,
+                                           self.version).visit(item)))
+
+        body.append(self.context.traceback.exit())
 
         if function_name == 'main':
             body, parameters = self.visit_function_definition_main(function,
