@@ -37,7 +37,7 @@ NUMBER_TYPES = INTEGER_TYPES | set(['f32', 'f64'])
 PRIMITIVE_TYPES = NUMBER_TYPES | set(['bool', 'char'])
 BUILTIN_TYPES = PRIMITIVE_TYPES | set(['string', 'bytes'])
 
-METHOD_OPERATORS = {
+METHOD_BIN_OPERATORS = {
     '__add__',
     '__sub__',
     '__mul__',
@@ -50,10 +50,19 @@ METHOD_OPERATORS = {
     '__le__'
 }
 
+METHOD_AUG_OPERATORS = {
+    '__iadd__',
+    '__isub__',
+    '__imul__',
+    '__idiv__'
+}
+
+METHOD_OPERATORS = METHOD_BIN_OPERATORS | METHOD_AUG_OPERATORS
+
 BUILTIN_ERRORS = {
-    'NoneError',
     'KeyError',
     'IndexError',
+    'InterruptError',
     'NotImplementedError',
     'AssertionError',
     'SystemExitError',
@@ -65,9 +74,12 @@ BUILTIN_CALLS = set(
     list(INTEGER_TYPES)
     + list(BUILTIN_ERRORS) + [
         'print',
+        'bytes',
         'char',
         'list',
         'input',
+        'set',
+        'regex',
         'str',
         'min',
         'max',
@@ -80,7 +92,12 @@ BUILTIN_CALLS = set(
         'reversed',
         'slice',
         'sum',
-        'zip'
+        'zip',
+        'string',
+        'default',
+        '__MYS_TRACEBACK_ENTER',
+        '__MYS_TRACEBACK_EXIT',
+        '__MYS_TRACEBACK_SET'
     ])
 
 OPERATORS = {
@@ -108,6 +125,15 @@ OPERATORS = {
     ast.GtE: '>='
 }
 
+COMPARISON_METHODS = {
+    ast.Eq: '__eq__',
+    ast.NotEq: '__ne__',
+    ast.Lt: '__lt__',
+    ast.LtE: '__le__',
+    ast.Gt: '__gt__',
+    ast.GtE: '__ge__'
+}
+
 OPERATORS_TO_METHOD = {
     ast.Add: '__add__',
     ast.Sub: '__sub__',
@@ -122,6 +148,26 @@ OPERATORS_TO_AUG_METHOD = {
     ast.Div: '__idiv__'
 }
 
+SET_METHODS = {
+    'is_disjoint': [['set'], 'bool'],
+    'is_subset': [['set'], 'bool'],
+    'is_proper_subset': [['set'], 'bool'],
+    'is_superset': [['set'], 'bool'],
+    'is_proper_superset': [['set'], 'bool'],
+    'union': [['set'], 'set'],
+    'update': [['set'], 'set'],
+    'intersection': [['set'], 'set'],
+    'intersection_update': [['set'], 'set'],
+    'difference': [['set'], 'set'],
+    'difference_update': [['set'], 'set'],
+    'symmetric_difference': [['set'], 'set'],
+    'symmetric_difference_update': [['set'], 'set'],
+    'add': [['elem'], None],
+    'clear': [[], None],
+    'discard': [['elem'], None],
+    'remove': [['elem'], None]
+}
+
 STRING_METHODS = {
     'to_utf8': [[], 'bytes'],
     'starts_with': [['string'], 'bool'],
@@ -143,9 +189,16 @@ STRING_METHODS = {
     'is_digit': [[], 'bool'],
     'is_numeric': [[], 'bool'],
     'is_space': [[], 'bool'],
+    'is_upper': [[], 'bool'],
+    'is_lower': [[], 'bool'],
     'match': [['regex'], 'regexmatch']
 }
 
+BYTES_METHODS = {
+    'to_hex': [[], 'string'],
+    'find': [['string', 'optional<i64>', 'optional<i64>'], 'i64']
+}
+
 LIST_METHODS = {
     'append': [['<listtype>'], None],
     'extend': [['list<listtype>'], None],
@@ -166,13 +219,20 @@ REGEX_METHODS = {
 
 REGEXMATCH_METHODS = {
     'span': [[None], ('i64', 'i64')],
-    'start': [[None], 'i64'],
+    'begin': [[None], 'i64'],
     'end': [[None], 'i64'],
     'group': [[None], 'string'],
     'groups': [[None], ['string']],
     'group_dict': [[], {'string': 'string'}]
 }
 
+CHAR_METHODS = {
+    'is_alpha': [[], 'bool'],
+    'is_digit': [[], 'bool'],
+    'is_numeric': [[], 'bool'],
+    'is_space': [[], 'bool']
+}
+
 
 def is_snake_case(value):
     return SNAKE_CASE_RE.match(value) is not None
@@ -193,6 +253,20 @@ def is_primitive_type(mys_type):
     return mys_type in PRIMITIVE_TYPES
 
 
+def is_integer_type(mys_type):
+    if not isinstance(mys_type, str):
+        return False
+
+    return mys_type in INTEGER_TYPES
+
+
+def is_float_type(mys_type):
+    if not isinstance(mys_type, str):
+        return False
+
+    return mys_type in ['f32', 'f64']
+
+
 def dot2ns(name):
     # Hack...
     if name not in BUILTIN_CALLS and name != 'Error':
@@ -258,7 +332,7 @@ def get_import_from_info(node, module_levels):
 
 
 def make_shared(cpp_type, values):
-    return f'std::make_shared<{cpp_type}>({values})'
+    return f'mys::make_shared<{cpp_type}>({values})'
 
 
 def shared_list_type(cpp_type):
@@ -266,12 +340,12 @@ def shared_list_type(cpp_type):
 
 
 def make_shared_list(cpp_type, value):
-    return (f'std::make_shared<List<{cpp_type}>>('
+    return (f'mys::make_shared<mys::List<{cpp_type}>>('
             f'std::initializer_list<{cpp_type}>{{{value}}})')
 
 
 def make_shared_set(cpp_type, value):
-    return (f'std::make_shared<Set<{cpp_type}>>('
+    return (f'mys::make_shared<Set<{cpp_type}>>('
             f'std::initializer_list<{cpp_type}>{{{value}}})')
 
 
@@ -280,8 +354,8 @@ def shared_dict_type(key_cpp_type, value_cpp_type):
 
 
 def make_shared_dict(key_cpp_type, value_cpp_type, items):
-    return (f'std::make_shared<Dict<{key_cpp_type}, {value_cpp_type}>>('
-            f'std::initializer_list<robin_hood::pair<{key_cpp_type}, '
+    return (f'mys::make_shared<Dict<{key_cpp_type}, {value_cpp_type}>>('
+            f'std::initializer_list<std::pair<{key_cpp_type}, '
             f'{value_cpp_type}>>{{{items}}})')
 
 
@@ -313,17 +387,19 @@ def mys_to_cpp_type(mys_type, context):
         return shared_set_type(item)
     else:
         if mys_type == 'string':
-            return 'String'
+            return 'mys::String'
         elif mys_type == 'bool':
-            return 'Bool'
+            return 'mys::Bool'
         elif mys_type == 'char':
-            return 'Char'
+            return 'mys::Char'
         elif mys_type == 'bytes':
-            return 'Bytes'
+            return 'mys::Bytes'
+        elif mys_type == 'regex':
+            return 'mys::Regex'
         elif mys_type == 'regexmatch':
-            return 'RegexMatch'
+            return 'mys::RegexMatch'
         elif context.is_class_or_trait_defined(mys_type):
-            return f'std::shared_ptr<{dot2ns(mys_type)}>'
+            return f'mys::shared_ptr<{dot2ns(mys_type)}>'
         elif context.is_enum_defined(mys_type):
             return context.get_enum_type(mys_type)
         else:
@@ -335,7 +411,7 @@ def mys_to_cpp_type_param(mys_type, context):
 
     if not is_primitive_type(mys_type):
         if not context.is_enum_defined(mys_type) and mys_type != 'string':
-            cpp_type = f'const {cpp_type}&'
+            cpp_type = f'{cpp_type}'
 
     return cpp_type
 
@@ -423,7 +499,7 @@ class MakeIntegerLiteralVisitor(ast.NodeVisitor):
         right = self.visit(node.right)
         op_class = type(node.op)
 
-        return format_binop(left, right, op_class)
+        return format_binop(left, right, op_class, True)
 
     def visit_UnaryOp(self, node):
         if isinstance(node.op, ast.USub):
@@ -497,8 +573,16 @@ def make_integer_literal(type_name, node):
 
 class MakeFloatLiteralVisitor(MakeIntegerLiteralVisitor):
 
+    def visit_BinOp(self, node):
+        left = self.visit(node.left)
+        right = self.visit(node.right)
+        op_class = type(node.op)
+
+        return format_binop(left, right, op_class, False)
+
     def visit_Constant(self, node):
         value = node.value * self.factor
+
         if self.type_name == 'f32':
             return str(value)
         elif self.type_name == 'f64':
@@ -519,9 +603,16 @@ def make_float_literal(type_name, node):
     return MakeFloatLiteralVisitor(type_name).visit(node)
 
 
-def format_binop(left, right, op_class):
+def format_binop(left, right, op_class, is_integer=True):
     if op_class == ast.Pow:
-        return f'ipow({left}, {right})'
+        if is_integer:
+            return f'ipow({left}, {right})'
+        else:
+            return f'std::pow({left}, {right})'
+    elif op_class in [ast.Mod, ast.Div]:
+        op = OPERATORS[op_class]
+
+        return f'({left} {op} mys::denominator_not_zero({right}))'
     else:
         op = OPERATORS[op_class]
 
@@ -549,6 +640,10 @@ def format_mys_type(mys_type):
     elif isinstance(mys_type, set):
         item = format_mys_type(list(mys_type)[0])
         return f'{{{item}}}'
+    elif isinstance(mys_type, GenericType):
+        types = ', '.join(format_mys_type(type) for type in mys_type.types)
+
+        return f'{mys_type.name}[{types}]'
     else:
         return str(mys_type)
 
