@@ -1,15 +1,18 @@
 from ..parser import ast
+from .definitions import GenericType
 from .generics import TypeVisitor
 from .generics import add_generic_class
 from .generics import fix_chosen_types
 from .generics import replace_generic_types
 from .utils import BUILTIN_CALLS
 from .utils import BUILTIN_ERRORS
+from .utils import BYTES_METHODS
 from .utils import LIST_METHODS
 from .utils import NUMBER_TYPES
 from .utils import OPERATORS_TO_METHOD
 from .utils import REGEX_METHODS
 from .utils import REGEXMATCH_METHODS
+from .utils import SET_METHODS
 from .utils import STRING_METHODS
 from .utils import CompileError
 from .utils import InternalError
@@ -22,7 +25,7 @@ from .utils import split_dict_mys_type
 
 def mys_to_value_type(mys_type):
     if isinstance(mys_type, tuple):
-        return tuple([mys_to_value_type(item) for item in mys_type])
+        return tuple(mys_to_value_type(item) for item in mys_type)
     elif isinstance(mys_type, list):
         return [mys_to_value_type(item) for item in mys_type]
     elif isinstance(mys_type, dict):
@@ -30,6 +33,8 @@ def mys_to_value_type(mys_type):
 
         return Dict(mys_to_value_type(key_mys_type),
                     mys_to_value_type(value_mys_type))
+    elif isinstance(mys_type, set):
+        return Set(mys_to_value_type(list(mys_type)[0]))
     else:
         return mys_type
 
@@ -91,7 +96,14 @@ def intersection_of(type_1, type_2, node):
                 new_type_2.append(item_type_2)
 
             return tuple(new_type_1), tuple(new_type_2)
-    elif isinstance(type_1, set):
+    elif isinstance(type_1, Set) and isinstance(type_2, Set):
+        value_type_1, value_type_2 = intersection_of(type_1.value_type,
+                                                     type_2.value_type,
+                                                     node)
+        return Set(value_type_1), Set(value_type_2)
+    elif isinstance(type_1, Set) and isinstance(type_2, Dict):
+        if type_2.key_type is not None or type_2.value_type is not None:
+            raise CompileError(f"cannot convert '{type_1}' to '{type_2}'", node)
         return type_1, type_1
     elif isinstance(type_1, Dict) and isinstance(type_2, Dict):
         if type_1.key_type is None and type_2.key_type is not None:
@@ -207,12 +219,20 @@ def reduce_type(value_type):
         return value_type
     elif isinstance(value_type, Dict):
         return {reduce_type(value_type.key_type): reduce_type(value_type.value_type)}
-    elif isinstance(value_type, set):
-        return value_type
-        #if len(value_type) == 1:
-        #    return {reduce_type(list(value_type)[0])}
-        #else:
-        #    return reduce_type(list(value_type)[0])
+    elif isinstance(value_type, Set):
+        return {reduce_type(value_type.value_type)}
+    elif value_type is None:
+        return None
+    else:
+        raise Exception("Bad reduce")
+
+
+class Set:
+    def __init__(self, value_type):
+        self.value_type = value_type
+
+    def __str__(self):
+        return f'Set({self.value_type})'
 
 
 class Dict:
@@ -281,7 +301,10 @@ class ValueTypeVisitor(ast.NodeVisitor):
         value_type = mys_to_value_type(self.visit(node.value))
 
         if isinstance(value_type, list):
-            value_type = value_type[0]
+            if isinstance(node.slice, ast.Slice):
+                pass
+            else:
+                value_type = value_type[0]
         elif isinstance(value_type, tuple):
             index = make_integer_literal('i64', node.slice)
             value_type = value_type[int(index)]
@@ -289,6 +312,7 @@ class ValueTypeVisitor(ast.NodeVisitor):
             value_type = value_type.value_type
         elif value_type == 'string':
             slice_type = self.visit(node.slice)
+
             if isinstance(slice_type, tuple):
                 value_type = 'string'
             else:
@@ -333,6 +357,8 @@ class ValueTypeVisitor(ast.NodeVisitor):
         if isinstance(value_type, dict):
             value_type = Dict(list(value_type.keys())[0],
                               list(value_type.values())[0])
+        elif isinstance(value_type, set):
+            value_type = Set(list(value_type)[0])
 
         return value_type
 
@@ -371,6 +397,8 @@ class ValueTypeVisitor(ast.NodeVisitor):
                 return 'char'
             else:
                 raise Exception('todo')
+        elif isinstance(node.value, complex):
+            raise CompileError('complex numbers are not supported', node)
         elif node.value is None:
             return None
         else:
@@ -387,6 +415,8 @@ class ValueTypeVisitor(ast.NodeVisitor):
             return 'string'
         elif name == '__file__':
             return 'string'
+        elif name == '__version__':
+            return 'string'
         elif self.context.is_local_variable_defined(name):
             value_type = self.context.get_local_variable_type(name)
 
@@ -410,7 +440,7 @@ class ValueTypeVisitor(ast.NodeVisitor):
         return [item_type]
 
     def visit_Tuple(self, node):
-        return tuple([self.visit(elem) for elem in node.elts])
+        return tuple(self.visit(elem) for elem in node.elts)
 
     def visit_Dict(self, node):
         if len(node.keys) > 0:
@@ -420,17 +450,14 @@ class ValueTypeVisitor(ast.NodeVisitor):
 
     def visit_Set(self, node):
         if len(node.elts) == 0:
-            return set([])
+            return Set([])
 
         item_type = self.visit(node.elts[0])
 
         for item in node.elts[1:]:
             item_type, _ = intersection_of(item_type, self.visit(item), item)
 
-        if isinstance(item_type, list):
-            return set(item_type)
-        else:
-            return set([item_type])
+        return Set(item_type)
 
     def visit_call_params_keywords(self, function, node):
         keyword_args = {}
@@ -574,16 +601,26 @@ class ValueTypeVisitor(ast.NodeVisitor):
         if name in NUMBER_TYPES:
             return name
         elif name == 'len':
-            return 'i64'
-        elif name == 'str':
+            return 'u64'
+        elif name in ['str', 'string']:
             return 'string'
         elif name == 'char':
             return 'char'
+        elif name == 'regex':
+            return 'regex'
+        elif name == 'set':
+            value_type = self.visit(node.args[0])
+            if isinstance(value_type, list):
+                return Set(value_type[0])
+            else:
+                raise InternalError(f"set('{value_type}') not supported", node)
         elif name == 'list':
             value_type = self.visit(node.args[0])
 
             if isinstance(value_type, Dict):
                 return [(value_type.key_type, value_type.value_type)]
+            elif value_type == 'string':
+                return ['char']
             else:
                 raise InternalError(f"list('{value_type}') not supported", node)
         elif name in ['min', 'max', 'sum']:
@@ -591,8 +628,8 @@ class ValueTypeVisitor(ast.NodeVisitor):
 
             if isinstance(value_type, list):
                 return value_type[0]
-            elif isinstance(value_type, set):
-                return list(value_type)[0]
+            elif isinstance(value_type, Set):
+                return value_type.value_type
             else:
                 return value_type
         elif name == 'abs':
@@ -615,6 +652,15 @@ class ValueTypeVisitor(ast.NodeVisitor):
             return 'string'
         elif name in BUILTIN_ERRORS:
             return name
+        elif name == 'bytes':
+            return 'bytes'
+        elif name == 'default':
+            type_name = node.args[0].id
+
+            if self.context.is_class_or_trait_defined(type_name):
+                return self.context.make_full_name(type_name)
+            else:
+                return type_name
         else:
             raise InternalError(f"builtin '{name}' not supported", node)
 
@@ -647,6 +693,25 @@ class ValueTypeVisitor(ast.NodeVisitor):
 
         return spec[1]
 
+    def visit_call_method_bytes(self, name, node):
+        spec = BYTES_METHODS.get(name, None)
+
+        if spec is None:
+            raise InternalError(f"bytes method '{name}' not supported", node)
+
+        return spec[1]
+
+    def visit_call_method_set(self, name, value_type, node):
+        spec = SET_METHODS.get(name, None)
+
+        if spec is None:
+            raise InternalError(f"set method '{name}' not supported", node)
+
+        if spec[1] == 'set':
+            return value_type
+        else:
+            return spec[1]
+
     def visit_call_method_regexmatch(self, name, node):
         spec = REGEXMATCH_METHODS.get(name, None)
 
@@ -681,12 +746,22 @@ class ValueTypeVisitor(ast.NodeVisitor):
 
         return returns
 
+    def visit_call_method_generic(self, name, value_type, node):
+        if self.context.is_class_defined(value_type.name):
+            value_type = add_generic_class(value_type.node, self.context)[1]
+
+            return self.visit_call_method_class(name, value_type, node)
+        else:
+            raise InternalError("generic trait not implemented", node)
+
     def visit_call_method(self, node):
         name = node.func.attr
         value_type = self.visit(node.func.value)
 
         if isinstance(value_type, list):
             return self.visit_call_method_list(name, value_type, node.func)
+        elif isinstance(value_type, Set):
+            return self.visit_call_method_set(name, value_type, node.func)
         elif isinstance(value_type, Dict):
             return self.visit_call_method_dict(name, value_type, node.func)
         elif value_type == 'string':
@@ -696,11 +771,13 @@ class ValueTypeVisitor(ast.NodeVisitor):
         elif value_type == 'regex':
             return self.visit_call_method_regex(name, node.func)
         elif value_type == 'bytes':
-            raise CompileError('bytes method not implemented', node.func)
+            return self.visit_call_method_bytes(name, node.func)
         elif self.context.is_class_defined(value_type):
             return self.visit_call_method_class(name, value_type, node)
         elif self.context.is_trait_defined(value_type):
             return self.visit_call_method_trait(name, value_type, node)
+        elif isinstance(value_type, GenericType):
+            return self.visit_call_method_generic(name, value_type, node)
         else:
             raise CompileError("None has no methods", node.func)
 
@@ -782,6 +859,8 @@ class ValueTypeVisitor(ast.NodeVisitor):
             item_type = iter_type[0]
         elif isinstance(iter_type, Dict):
             item_type = (iter_type.key_type, iter_type.value_type)
+        elif iter_type == 'string':
+            item_type = 'char'
         else:
             raise CompileError("unsupported type", node)
 
@@ -814,4 +893,20 @@ class ValueTypeVisitor(ast.NodeVisitor):
         return result_type
 
     def visit_SetComp(self, node):
-        raise CompileError("set comprehension is not implemented", node)
+        if len(node.generators) != 1:
+            raise CompileError("only one for-loop allowed", node)
+
+        generator = node.generators[0]
+        iter_type = self.visit(generator.iter)
+
+        if isinstance(iter_type, list):
+            item_type = list(iter_type)[0]
+        else:
+            raise CompileError(f"unsupported type '{iter_type}'", node)
+
+        self.context.push()
+        self.visit_define_local_variables(generator.target, item_type)
+        result_type = Set(self.visit(node.elt))
+        self.context.pop()
+
+        return result_type
