@@ -1,6 +1,8 @@
 import os
 import shutil
 import subprocess
+import tarfile
+from io import BytesIO
 from io import StringIO
 from unittest.mock import Mock
 from unittest.mock import call
@@ -11,7 +13,6 @@ import mys.cli
 from .utils import Path
 from .utils import TestCase
 from .utils import create_new_package
-from .utils import read_file
 from .utils import remove_ansi
 from .utils import remove_build_directory
 from .utils import run_mys_command
@@ -19,11 +20,6 @@ from .utils import run_mys_command
 
 class Test(TestCase):
 
-    def assert_files_equal(self, actual, expected):
-        # os.makedirs(os.path.dirname(expected), exist_ok=True)
-        # open(expected, 'w').write(open(actual, 'r').read())
-        self.assertEqual(read_file(actual), read_file(expected))
-
     def test_foo_new_and_run(self):
         package_name = 'test_foo_new_and_run'
         remove_build_directory(package_name)
@@ -64,6 +60,8 @@ class Test(TestCase):
                                 'tests/files/foo/src/main.mys')
         self.assert_files_equal(f'tests/build/{package_name}/src/lib.mys',
                                 'tests/files/foo/src/lib.mys')
+        self.assert_files_equal(f'tests/build/{package_name}/doc/index.rst',
+                                'tests/files/foo/doc/index.rst')
 
         with Path(f'tests/build/{package_name}'):
             # Run.
@@ -73,8 +71,8 @@ class Test(TestCase):
                 mys.cli.main()
 
             self.assert_file_exists(
-                f'build/default/cpp/src/{package_name}/main.mys.cpp')
-            self.assert_file_exists('build/default/app')
+                f'build/speed/cpp/src/{package_name}/main.mys.cpp')
+            self.assert_file_exists('build/speed/app')
 
             # Clean.
             self.assert_file_exists('build')
@@ -91,7 +89,7 @@ class Test(TestCase):
             with patch('sys.argv', ['mys', '-d', 'build', '-j', '1']):
                 mys.cli.main()
 
-            self.assert_file_exists('./build/default/app')
+            self.assert_file_exists('./build/speed/app')
 
             # Run again, but with run() mock to verify that the
             # application is run.
@@ -108,24 +106,24 @@ class Test(TestCase):
                 run_mock.mock_calls,
                 [
                     [
-                        call(['make', '-f', 'build/default/Makefile', 'all',
+                        call(['make', '-f', 'build/speed/Makefile', 'all',
                               '-s', 'APPLICATION=yes'],
                              stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT,
                              encoding='utf-8',
                              close_fds=False,
                              env=None),
-                        call(['./build/default/app'], check=True)
+                        call(['./build/speed/app'], check=True)
                     ],
                     [
-                        call(['make', '-f', 'build/default/Makefile', 'all',
+                        call(['make', '-f', 'build/speed/Makefile', 'all',
                               '-j', '1', '-s', 'APPLICATION=yes'],
                              stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT,
                              encoding='utf-8',
                              close_fds=False,
                              env=None),
-                        call(['./build/default/app'], check=True)
+                        call(['./build/speed/app'], check=True)
                     ]
                 ])
 
@@ -133,7 +131,26 @@ class Test(TestCase):
             with patch('sys.argv', ['mys', '-d', 'test', '-j', '1']):
                 mys.cli.main()
 
-            self.assert_file_exists('./build/default/test')
+            self.assert_file_exists('./build/debug/test')
+
+            # Doc without logo.
+            with patch('sys.argv', ['mys', '-d', 'doc']):
+                mys.cli.main()
+
+            self.assert_file_exists('./build/doc/html/index.html')
+            self.assert_file_not_exists('./build/doc/html/_static/logo.png')
+
+            # Doc with logo.
+            os.makedirs('doc/images')
+
+            with open('doc/images/logo.png', 'w'):
+                pass
+
+            with patch('sys.argv', ['mys', 'doc']):
+                mys.cli.main()
+
+            self.assert_file_exists('./build/doc/html/index.html')
+            self.assert_file_exists('./build/doc/html/_static/logo.png')
 
     def test_new_author_from_git(self):
         package_name = 'test_new_author_from_git'
@@ -203,36 +220,40 @@ class Test(TestCase):
         create_new_package(package_name)
 
         with Path(f'tests/build/{package_name}'):
-            # Publish.
-            run_sdist_result = Mock()
-            run_twine_result = Mock()
-            run_mock = Mock(side_effect=[run_sdist_result, run_twine_result])
+            os.makedirs('assets')
 
-            with patch('sys.argv', ['mys', '-d', 'publish', '-u', 'a', '-p', 'b']):
-                with patch('subprocess.run', run_mock):
+            with open('assets/foo.txt', 'w'):
+                pass
+
+            def post_mock(url, params, data):
+                del params
+                self.assertEqual(
+                    url,
+                    'https://mys-lang.org/package/test_publish-0.1.0.tar.gz')
+
+                with tarfile.open(fileobj=BytesIO(data), mode='r:gz') as fin:
+                    names = fin.getnames()
+
+                self.assertEqual(names,
+                                 [
+                                     'test_publish-0.1.0',
+                                     'test_publish-0.1.0/README.rst',
+                                     'test_publish-0.1.0/assets',
+                                     'test_publish-0.1.0/assets/foo.txt',
+                                     'test_publish-0.1.0/doc',
+                                     'test_publish-0.1.0/doc/index.rst',
+                                     'test_publish-0.1.0/package.toml',
+                                     'test_publish-0.1.0/src',
+                                     'test_publish-0.1.0/src/lib.mys',
+                                     'test_publish-0.1.0/src/main.mys'
+                                 ])
+
+                return Mock(status_code=200, text=None)
+
+            with patch('sys.argv', ['mys', '-d', 'publish']):
+                with patch('requests.post', post_mock):
                     mys.cli.main()
 
-            # sdist.
-            the_call = run_mock.call_args_list[0]
-            self.assertEqual(the_call.args[0][1:], ['setup.py', 'sdist'])
-            self.assertEqual(the_call.kwargs,
-                             {
-                                 'stdout': subprocess.PIPE,
-                                 'stderr': subprocess.STDOUT,
-                                 'encoding': 'utf-8',
-                                 'close_fds': False,
-                                 'env': None
-                             })
-
-            # twine.
-            the_call = run_mock.call_args_list[1]
-            self.assertEqual(the_call.args[0][1:], ['-m', 'twine', 'upload'])
-            self.assertEqual(the_call.kwargs['stdout'], subprocess.PIPE)
-            self.assertEqual(the_call.kwargs['stderr'], subprocess.STDOUT)
-            self.assertEqual(the_call.kwargs['encoding'], 'utf-8')
-            self.assertEqual(the_call.kwargs['env']['TWINE_USERNAME'], 'a')
-            self.assertEqual(the_call.kwargs['env']['TWINE_PASSWORD'], 'b')
-
     def test_build_outside_package(self):
         # Empty directory.
         package_name = 'test_build_outside_package'
@@ -331,6 +352,8 @@ class Test(TestCase):
                                   capture_output=True,
                                   text=True)
             self.assertEqual(proc.stdout, "hello\n")
+            self.assertTrue(os.path.exists('../bin/foo-assets/foo/foo.json'))
+            self.assertTrue(os.path.exists('../bin/foo-assets/bar/bar.txt'))
 
     def test_install_package_from_registry(self):
         package_name = 'test_install_package_from_registry'
@@ -387,3 +410,48 @@ class Test(TestCase):
 
         self.assertEqual(str(cm.exception),
                          "package name must be snake case, got 'a-hypen'")
+
+    def test_error_if_package_version_is_not_semantic(self):
+        name = 'test_error_if_package_version_is_not_semantic'
+        remove_build_directory(name)
+        shutil.copytree('tests/files/error_if_package_version_is_not_semantic',
+                        f'tests/build/{name}')
+
+        with Path(f'tests/build/{name}'):
+            with self.assertRaises(SystemExit) as cm:
+                with patch('sys.argv', ['mys', 'build']):
+                    mys.cli.main()
+
+        self.assertEqual(str(cm.exception),
+                         "package version must be a semantic version, got '1.0'")
+
+    def test_package_with_local_and_registry_dependencies(self):
+        name = 'test_package_with_local_and_registry_dependencies'
+        remove_build_directory(name)
+        shutil.copytree('tests/files/package_with_local_and_registry_dependencies',
+                        f'tests/build/{name}')
+
+        with Path(f'tests/build/{name}/top'):
+            # Default.
+            stdout = StringIO()
+
+            with patch('sys.stdout', stdout):
+                with patch('sys.argv', ['mys', 'dependencies']):
+                    mys.cli.main()
+
+            self.assert_in('dep1 = { path = "../dep1" }\n'
+                           'hello_world = "latest"\n'
+                           'dep2 = { path = "../dep2" }\n',
+                           stdout.getvalue())
+
+            # Versions.
+            stdout = StringIO()
+
+            with patch('sys.stdout', stdout):
+                with patch('sys.argv', ['mys', 'dependencies', '--versions']):
+                    mys.cli.main()
+
+            self.assert_in('dep1 = "1.1.0"\n'
+                           'hello_world = "0.4.0"\n'
+                           'dep2 = "0.1.0-rc10"\n',
+                           stdout.getvalue())
