@@ -93,26 +93,6 @@ class Test(TestCase):
             '            ^\n'
             "CompileError: expected a 'string', got a 'bool'\n")
 
-    def test_compare_i64_and_bool(self):
-        self.assert_transpile_raises(
-            'def foo() -> bool:\n'
-            '    return 1 == True',
-            '  File "", line 2\n'
-            '        return 1 == True\n'
-            '               ^\n'
-            "CompileError: cannot convert 'i64/i32/i16/i8/u64/u32/u16/u8' "
-            "to 'bool'\n")
-
-    def test_compare_mix_of_literals_and_known_types_1(self):
-        source = transpile_source('def foo():\n'
-                                  '    k: u64 = 1\n'
-                                  '    v: i64 = 1\n'
-                                  '    if 0xffffffffffffffff == k:\n'
-                                  '        pass\n'
-                                  '    print(v)\n')
-
-        self.assert_in('18446744073709551615ull', source)
-
     def test_assign_256_to_u8(self):
         self.assert_transpile_raises(
             'A: u8 = 256\n',
@@ -150,15 +130,6 @@ class Test(TestCase):
             '            ^\n'
             "CompileError: cannot convert float to 'u8'\n")
 
-    def test_arithmetics_on_mix_of_literals_and_known_types_5(self):
-        source = transpile_source('def foo():\n'
-                                  '    k: i32 = -1\n'
-                                  '    v: u8 = 1\n'
-                                  '    value = ((-1 / 2) - 2 * k)\n'
-                                  '    print(value, v)\n')
-
-        self.assert_in('value = ((-1 / 2) - (2 * k));', source)
-
     def test_assign_negative_number_to_u32(self):
         self.assert_transpile_raises(
             'def foo():\n'
@@ -210,20 +181,12 @@ class Test(TestCase):
             '                     ^\n'
             "CompileError: integer literal out of range for 'u8'\n")
 
-    def test_arithmetics_and_compare(self):
-        source = transpile_source('def foo():\n'
-                                  '    k: i32 = -1\n'
-                                  '    if ((-1 / 2) - 2 * k) == k:\n'
-                                  '        pass\n')
-
-        self.assert_in('if (Bool(((-1 / 2) - (2 * k)) == k)) {', source)
-
     def test_change_integer_type(self):
         source = transpile_source('def foo():\n'
                                   '    value = (i8(-1) * i8(u32(5)))\n'
                                   '    print(value)\n')
 
-        self.assert_in('value = (i8(-1) * i8(u32(5)));', source)
+        self.assert_in('value = (i8(-(1)) * i8(u32(5)));', source)
 
     def test_change_integer_type_error(self):
         self.assert_transpile_raises(
@@ -267,26 +230,6 @@ class Test(TestCase):
             '               ^\n'
             "CompileError: only tuples can be unpacked\n")
 
-    def test_assert_between(self):
-        self.assert_transpile_raises(
-            'def foo():\n'
-            '    a = 2\n'
-            '    assert 1 <= a < 3\n',
-            '  File "", line 3\n'
-            "        assert 1 <= a < 3\n"
-            '               ^\n'
-            "CompileError: can only compare two values\n")
-
-    def test_compare_between(self):
-        self.assert_transpile_raises(
-            'def foo():\n'
-            '    a = 2\n'
-            '    print(1 <= a < 3)\n',
-            '  File "", line 3\n'
-            "        print(1 <= a < 3)\n"
-            '              ^\n'
-            "CompileError: can only compare two values\n")
-
     def test_type_error_1(self):
         self.assert_transpile_raises(
             'def foo():\n'
@@ -351,183 +294,6 @@ class Test(TestCase):
             '               ^\n'
             "CompileError: function does not return any value\n")
 
-    def test_compare_wrong_types_1(self):
-        self.assert_transpile_raises(
-            'def foo() -> bool:\n'
-            '    return 1 == [""]\n',
-            '  File "", line 2\n'
-            '        return 1 == [""]\n'
-            '               ^\n'
-            "CompileError: cannot convert 'i64/i32/i16/i8/u64/u32/u16/u8' to "
-            "'[string]'\n")
-
-    def test_compare_wrong_types_2(self):
-        self.assert_transpile_raises(
-            'def foo() -> bool:\n'
-            '    return [""] in 1\n',
-            '  File "", line 2\n'
-            '        return [""] in 1\n'
-            '                       ^\n'
-            "CompileError: not an iterable\n")
-
-    def test_compare_wrong_types_3(self):
-        self.assert_transpile_raises(
-            'def foo() -> bool:\n'
-            '    return [""] not in 1\n',
-            '  File "", line 2\n'
-            '        return [""] not in 1\n'
-            '                           ^\n'
-            "CompileError: not an iterable\n")
-
-    def test_compare_wrong_types_4(self):
-        self.assert_transpile_raises(
-            'def foo() -> bool:\n'
-            '    return 2.0 == 1\n',
-            '  File "", line 2\n'
-            '        return 2.0 == 1\n'
-            '               ^\n'
-            "CompileError: cannot convert 'f64/f32' to "
-            "'i64/i32/i16/i8/u64/u32/u16/u8'\n")
-
-    def test_compare_wrong_types_5(self):
-        self.assert_transpile_raises(
-            'def foo() -> bool:\n'
-            '    return 1.0 == [""]\n',
-            '  File "", line 2\n'
-            '        return 1.0 == [""]\n'
-            '               ^\n'
-            "CompileError: cannot convert 'f64/f32' to '[string]'\n")
-
-    def test_compare_wrong_types_6(self):
-        self.assert_transpile_raises(
-            'def foo(a: i32) -> bool:\n'
-            '    return a in [""]\n',
-            '  File "", line 2\n'
-            '        return a in [""]\n'
-            '               ^\n'
-            "CompileError: types 'i32' and 'string' differs\n")
-
-    def test_compare_wrong_types_7(self):
-        self.assert_transpile_raises(
-            'def foo(a: i32) -> bool:\n'
-            '    return a in a\n',
-            '  File "", line 2\n'
-            '        return a in a\n'
-            '                    ^\n'
-            "CompileError: not an iterable\n")
-
-    def test_compare_wrong_types_8(self):
-        self.assert_transpile_raises(
-            'def foo(a: i32) -> bool:\n'
-            '    return 1 in a\n',
-            '  File "", line 2\n'
-            '        return 1 in a\n'
-            '                    ^\n'
-            "CompileError: not an iterable\n")
-
-    def test_compare_wrong_types_9(self):
-        self.assert_transpile_raises(
-            'def foo(a: i32) -> bool:\n'
-            '    return "" == a\n',
-            '  File "", line 2\n'
-            '        return "" == a\n'
-            '               ^\n'
-            "CompileError: types 'string' and 'i32' differs\n")
-
-    def test_compare_wrong_types_10(self):
-        self.assert_transpile_raises(
-            'def foo():\n'
-            '    print(1 is None)\n',
-            '  File "", line 2\n'
-            '        print(1 is None)\n'
-            '                   ^\n'
-            "CompileError: 'i64' cannot be None\n")
-
-    def test_compare_wrong_types_11(self):
-        self.assert_transpile_raises(
-            'def foo():\n'
-            '    print(1.0 is None)\n',
-            '  File "", line 2\n'
-            '        print(1.0 is None)\n'
-            '                     ^\n'
-            "CompileError: 'f64' cannot be None\n")
-
-    def test_compare_wrong_types_12(self):
-        self.assert_transpile_raises(
-            'def foo(a: i32):\n'
-            '    print(a is None)\n',
-            '  File "", line 2\n'
-            '        print(a is None)\n'
-            '              ^\n'
-            "CompileError: 'i32' cannot be None\n")
-
-    def test_compare_wrong_types_13(self):
-        self.assert_transpile_raises(
-            'def foo(a: i32):\n'
-            '    print(None is a)\n',
-            '  File "", line 2\n'
-            '        print(None is a)\n'
-            '              ^\n'
-            "CompileError: 'i32' cannot be None\n")
-
-    def test_compare_wrong_types_14(self):
-        self.assert_transpile_raises(
-            'def foo():\n'
-            '    print(True is None)\n',
-            '  File "", line 2\n'
-            '        print(True is None)\n'
-            '              ^\n'
-            "CompileError: 'bool' cannot be None\n")
-
-    def test_compare_wrong_types_15(self):
-        self.assert_transpile_raises(
-            'def foo(a: bool):\n'
-            '    print(None is a)\n',
-            '  File "", line 2\n'
-            '        print(None is a)\n'
-            '              ^\n'
-            "CompileError: 'bool' cannot be None\n")
-
-    def test_compare_wrong_types_16(self):
-        self.assert_transpile_raises(
-            'def foo(a: bool):\n'
-            '    print(a is not 1)\n',
-            '  File "", line 2\n'
-            '        print(a is not 1)\n'
-            '              ^\n'
-            "CompileError: cannot convert 'bool' to "
-            "'i64/i32/i16/i8/u64/u32/u16/u8'\n")
-
-    def test_compare_wrong_types_17(self):
-        self.assert_transpile_raises(
-            'def foo():\n'
-            '    print(None in [1, 5])\n',
-            '  File "", line 2\n'
-            '        print(None in [1, 5])\n'
-            '              ^\n'
-            "CompileError: 'i64' cannot be None\n")
-
-    def test_compare_wrong_types_18(self):
-        self.assert_transpile_raises(
-            'def foo():\n'
-            '    print(None == "")\n',
-            '  File "", line 2\n'
-            '        print(None == "")\n'
-            '              ^\n'
-            "CompileError: use 'is' and 'is not' to compare to None\n")
-
-    def test_compare_wrong_types_20(self):
-        self.assert_transpile_raises(
-            'def foo():\n'
-            '    if (1, ("", True)) == (1, ("", 1)):\n'
-            '        pass\n',
-            # ToDo: Marker in wrong place.
-            '  File "", line 2\n'
-            '        if (1, ("", True)) == (1, ("", 1)):\n'
-            '           ^\n'
-            "CompileError: cannot convert 'bool' to "
-            "'i64/i32/i16/i8/u64/u32/u16/u8'\n")
-
     def test_bool_op_1(self):
         self.assert_transpile_raises(
             'def foo():\n'
@@ -563,6 +329,8 @@ class Test(TestCase):
 
         self.assert_in('void foo(void)\n'
                        '{\n'
+                       '    __MYS_TRACEBACK_ENTER();\n'
+                       '    __MYS_TRACEBACK_EXIT();\n'
                        '}\n',
                        source)
 
@@ -592,14 +360,6 @@ class Test(TestCase):
             '            ^\n'
             "CompileError: expected a 'i64', got a 'string'\n")
 
-    def test_format_string_i8_u8(self):
-        source = transpile_source('def foo():\n'
-                                  '    print(f"{i8(1)} {u8(1)} {u16(1)}")\n')
-
-        self.assert_in('String((int)i8(1))', source)
-        self.assert_in('String((unsigned)u8(1))', source)
-        self.assert_in('String(u16(1))', source)
-
     def test_min_wrong_types(self):
         self.assert_transpile_raises(
             'def foo():\n'
@@ -679,7 +439,7 @@ class Test(TestCase):
             '  File "", line 2\n'
             '        assert len("") == i8(0)\n'
             '               ^\n'
-            "CompileError: types 'i64' and 'i8' differs\n")
+            "CompileError: types 'u64' and 'i8' differs\n")
 
     def test_str_compare_to_non_string(self):
         self.assert_transpile_raises(
@@ -847,15 +607,6 @@ class Test(TestCase):
             '              ^\n'
             "CompileError: None cannot be printed\n")
 
-    def test_bare_compare(self):
-        self.assert_transpile_raises(
-            'def foo():\n'
-            '    1 == 2\n',
-            '  File "", line 2\n'
-            '        1 == 2\n'
-            '        ^\n'
-            "CompileError: bare comparision\n")
-
     def test_bare_integer(self):
         self.assert_transpile_raises(
             'def foo():\n'
@@ -996,7 +747,7 @@ class Test(TestCase):
                                   'def bar():\n'
                                   '    foo()\n')
 
-        self.assert_in('foo(Bool(true))', source)
+        self.assert_in('foo(mys::Bool(true))', source)
 
     def test_inline_constant_default_u8_parameter_value(self):
         source = transpile_source('def foo(a: u8 = 1):\n'
@@ -1028,7 +779,7 @@ class Test(TestCase):
                                   'def bar():\n'
                                   '    foo()\n')
 
-        self.assert_in('foo(String("hi"))', source)
+        self.assert_in('foo(mys::String("hi"))', source)
 
     def test_inline_constant_default_class_parameter_value_none(self):
         source = transpile_source('class Foo:\n'
@@ -1068,15 +819,6 @@ class Test(TestCase):
             '             ^\n'
             "CompileError: expected a 'i32', got a 'i8'\n")
 
-    def test_complex(self):
-        # complex may be implemented at some point.
-        self.assert_transpile_raises(
-            'VAR: complex = 1 + 2j\n',
-            '  File "", line 1\n'
-            '    VAR: complex = 1 + 2j\n'
-            '         ^\n'
-            "CompileError: undefined type 'complex'\n")
-
     def test_undefined_type_as_function_parameter(self):
         self.assert_transpile_raises(
             'def add(a: Foo):\n'
@@ -1095,6 +837,16 @@ class Test(TestCase):
             '                 ^\n'
             "CompileError: undefined type 'Foo'\n")
 
+    def test_power_f64_and_i64(self):
+        self.assert_transpile_raises(
+            'def foo():\n'
+            '    print(2.0 ** 3)\n',
+            '  File "", line 2\n'
+            '        print(2.0 ** 3)\n'
+            '              ^\n'
+            "CompileError: cannot convert 'f64/f32' to "
+            "'i64/i32/i16/i8/u64/u32/u16/u8'\n")
+
     # ToDo
     # def test_method_name_same_as_member_name(self):
     #     self.assert_transpile_raises(
